@@ -74,6 +74,8 @@ enum {
 	SIGNAL_SIZE_PREPARED,
 	SIGNAL_THUMBNAIL_CHANGED,
 	SIGNAL_SAVE_PROGRESS,
+	SIGNAL_NEXT_FRAME,
+	SIGNAL_FILE_CHANGED,
 	SIGNAL_LAST
 };
 
@@ -93,11 +95,30 @@ eog_image_free_mem_private (EogImage *image)
 	if (priv->status == EOG_IMAGE_STATUS_LOADING) {
 		eog_image_cancel_load (image);
 	} else {
+		if (priv->anim_iter != NULL) {
+			g_object_unref (priv->anim_iter);
+			priv->anim_iter = NULL;
+		}
+
+		if (priv->anim != NULL) {
+			g_object_unref (priv->anim);
+			priv->anim = NULL;
+		}
+
+		priv->is_playing = FALSE;
+
 		if (priv->image != NULL) {
 			g_object_unref (priv->image);
 			priv->image = NULL;
 		}
 
+#ifdef HAVE_RSVG
+		if (priv->svg != NULL) {
+			g_object_unref (priv->svg);
+			priv->svg = NULL;
+		}
+#endif
+
 #ifdef HAVE_EXIF
 		if (priv->exif != NULL) {
 			exif_data_unref (priv->exif);
@@ -228,6 +249,31 @@ eog_image_class_init (EogImageClass *klass)
 			      g_cclosure_marshal_VOID__FLOAT,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_FLOAT);
+ 	/**
+ 	 * EogImage::next-frame:
+  	 * @img: the object which received the signal.
+	 * @delay: number of milliseconds the current frame will be displayed.
+	 *
+	 * The ::next-frame signal will be emitted each time an animated image
+	 * advances to the next frame.
+	 */
+	signals[SIGNAL_NEXT_FRAME] =
+		g_signal_new ("next-frame",
+			      EOG_TYPE_IMAGE,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EogImageClass, next_frame),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__INT,
+			      G_TYPE_NONE, 1,
+			      G_TYPE_INT);
+
+	signals[SIGNAL_FILE_CHANGED] = g_signal_new ("file-changed",
+						     EOG_TYPE_IMAGE,
+						     G_SIGNAL_RUN_LAST,
+						     G_STRUCT_OFFSET (EogImageClass, file_changed),
+						     NULL, NULL,
+						     g_cclosure_marshal_VOID__VOID,
+						     G_TYPE_NONE, 0);
 
 	g_type_class_add_private (object_class, sizeof (EogImagePrivate));
 }
@@ -239,6 +285,9 @@ eog_image_init (EogImage *img)
 
 	img->priv->file = NULL;
 	img->priv->image = NULL;
+	img->priv->anim = NULL;
+	img->priv->anim_iter = NULL;
+	img->priv->is_playing = FALSE;
 	img->priv->thumbnail = NULL;
 	img->priv->width = -1;
 	img->priv->height = -1;
@@ -262,6 +311,9 @@ eog_image_init (EogImage *img)
 #ifdef HAVE_LCMS
 	img->priv->profile = NULL;
 #endif
+#ifdef HAVE_RSVG
+	img->priv->svg = NULL;
+#endif
 }
 
 EogImage *
@@ -713,7 +765,7 @@ eog_image_set_orientation (EogImage *img)
 static void
 eog_image_real_autorotate (EogImage *img)
 {
-	static EogTransformType lookup[8] = {EOG_TRANSFORM_NONE,
+	static const EogTransformType lookup[8] = {EOG_TRANSFORM_NONE,
 					     EOG_TRANSFORM_FLIP_HORIZONTAL,
 					     EOG_TRANSFORM_ROT_180,
 					     EOG_TRANSFORM_FLIP_VERTICAL,
@@ -845,7 +897,8 @@ eog_image_real_load (EogImage *img,
 	gboolean set_metadata = TRUE;
 	gboolean read_image_data = (data2read & EOG_IMAGE_DATA_IMAGE);
 	gboolean read_only_dimension = (data2read & EOG_IMAGE_DATA_DIMENSION) &&
-					!read_image_data;
+				  ((data2read ^ EOG_IMAGE_DATA_DIMENSION) == 0);
+
 
 	priv = img->priv;
 
@@ -902,6 +955,21 @@ eog_image_real_load (EogImage *img,
 	if (read_image_data || read_only_dimension) {
 		gboolean checked_threadsafety = FALSE;
 
+#ifdef HAVE_RSVG
+		if (priv->svg != NULL) {
+			g_object_unref (priv->svg);
+			priv->svg = NULL;
+		}
+
+		if (!strcmp (mime_type, "image/svg+xml")) {
+			gchar *file_path;
+			/* Keep the object for rendering */
+			priv->svg = rsvg_handle_new ();
+			file_path = g_file_get_path (priv->file);
+			rsvg_handle_set_base_uri (priv->svg, file_path);
+			g_free (file_path);
+		}
+#endif
 		loader = gdk_pixbuf_loader_new_with_mime_type (mime_type, error);
 
 		if (error && *error) {
@@ -953,10 +1021,18 @@ eog_image_real_load (EogImage *img,
 			break;
 		}
 
-		if ((read_image_data || read_only_dimension) &&
-		    !gdk_pixbuf_loader_write (loader, buffer, bytes_read, error)) {
-			failed = TRUE;
-			break;
+		if ((read_image_data || read_only_dimension)) {
+			if (!gdk_pixbuf_loader_write (loader, buffer, bytes_read, error)) {
+				failed = TRUE;
+				break;
+			}
+#ifdef HAVE_RSVG
+			if (eog_image_is_svg (img) &&
+			    !rsvg_handle_write (priv->svg, buffer, bytes_read, error)) {
+				failed = TRUE;
+				break;
+			}
+#endif
 		}
 
 		bytes_read_total += bytes_read;
@@ -1026,6 +1102,10 @@ eog_image_real_load (EogImage *img,
 				g_clear_error (error);
 			}
 	        }
+#ifdef HAVE_RSVG
+		if (eog_image_is_svg (img))
+			rsvg_handle_close (priv->svg, error);
+#endif
         }
 
 	g_free (buffer);
@@ -1049,7 +1129,15 @@ eog_image_real_load (EogImage *img,
 			g_object_unref (priv->image);
 		}
 
-		priv->image = gdk_pixbuf_loader_get_pixbuf (loader);
+		priv->anim = gdk_pixbuf_loader_get_animation (loader);
+
+		if (gdk_pixbuf_animation_is_static_image (priv->anim)) {
+			priv->image = gdk_pixbuf_animation_get_static_image (priv->anim);
+			priv->anim = NULL;
+		} else {
+			priv->anim_iter = gdk_pixbuf_animation_get_iter (priv->anim,NULL);
+			priv->image = gdk_pixbuf_animation_iter_get_pixbuf (priv->anim_iter);
+		}
 
 		if (G_LIKELY (priv->image != NULL)) {
 			g_object_ref (priv->image);
@@ -1164,7 +1252,10 @@ eog_image_load (EogImage *img, EogImageData data2read, EogJob *job, GError **err
 	success = eog_image_real_load (img, data2read, job, error);
 
 #ifdef HAVE_EXIF
-	if (priv->autorotate) {
+	/* Check that the metadata was loaded at least once before
+	 * trying to autorotate. */
+	if (priv->autorotate && 
+	    priv->metadata_status == EOG_IMAGE_METADATA_READY) {
 		eog_image_real_autorotate (img);
 	}
 #endif
@@ -2007,3 +2098,140 @@ eog_image_is_supported_mime_type (const char *mime_type)
 
 	return (result != NULL);
 }
+
+static gboolean
+eog_image_iter_advance (EogImage *img)
+{
+	EogImagePrivate *priv;
+ 	gboolean new_frame;
+
+	g_return_val_if_fail (EOG_IS_IMAGE (img), FALSE);
+	g_return_val_if_fail (GDK_IS_PIXBUF_ANIMATION_ITER (img->priv->anim_iter), FALSE);
+
+	priv = img->priv;
+
+	if ((new_frame = gdk_pixbuf_animation_iter_advance (img->priv->anim_iter, NULL)) == TRUE)
+	  {      
+		g_mutex_lock (priv->status_mutex);
+		g_object_unref (priv->image);
+		priv->image = gdk_pixbuf_animation_iter_get_pixbuf (priv->anim_iter);
+	 	g_object_ref (priv->image);
+		/* keep the transformation over time */
+		if (EOG_IS_TRANSFORM (priv->trans)) {
+			GdkPixbuf* transformed = eog_transform_apply (priv->trans, priv->image, NULL);
+			g_object_unref (priv->image);
+			priv->image = transformed;
+			priv->width = gdk_pixbuf_get_width (transformed);
+			priv->height = gdk_pixbuf_get_height (transformed);
+		}      
+		g_mutex_unlock (priv->status_mutex);
+		/* Emit next frame signal so we can update the display */
+		g_signal_emit (img, signals[SIGNAL_NEXT_FRAME], 0,
+			       gdk_pixbuf_animation_iter_get_delay_time (priv->anim_iter));
+	  }
+
+	return new_frame;
+}
+
+/**
+ * eog_image_is_animation:
+ * @img: a #EogImage
+ *
+ * Checks whether a given image is animated.
+ *
+ * Returns: #TRUE if it is an animated image, #FALSE otherwise.
+ * 
+ **/
+gboolean
+eog_image_is_animation (EogImage *img)
+{
+	g_return_val_if_fail (EOG_IS_IMAGE (img), FALSE);
+	return img->priv->anim != NULL;
+}
+
+static gboolean
+private_timeout (gpointer data)
+{
+	EogImage *img = EOG_IMAGE (data);
+	EogImagePrivate *priv = img->priv;
+
+	if (eog_image_is_animation (img) && 
+	    !g_source_is_destroyed (g_main_current_source ()) &&
+	    priv->is_playing) {
+		while (eog_image_iter_advance (img) != TRUE) {}; /* cpu-sucking ? */
+			g_timeout_add (gdk_pixbuf_animation_iter_get_delay_time (priv->anim_iter), private_timeout, img);
+	 		return FALSE;
+ 	}
+	priv->is_playing = FALSE;
+	return FALSE; /* stop playing */
+}
+
+/**
+ * eog_image_start_animation:
+ * @img: a #EogImage
+ *
+ * Starts playing an animated image.
+ *
+ * Returns: %TRUE on success, %FALSE if @img is already playing or isn't an animated image.
+ **/
+gboolean
+eog_image_start_animation (EogImage *img)
+{
+	EogImagePrivate *priv;
+
+	g_return_val_if_fail (EOG_IS_IMAGE (img), FALSE);
+	priv = img->priv;
+
+	if (!eog_image_is_animation (img) || priv->is_playing)
+		return FALSE;
+
+	g_mutex_lock (priv->status_mutex);
+	g_object_ref (priv->anim_iter);
+	priv->is_playing = TRUE;
+	g_mutex_unlock (priv->status_mutex);
+
+ 	g_timeout_add (gdk_pixbuf_animation_iter_get_delay_time (priv->anim_iter), private_timeout, img);
+
+	return TRUE;
+}
+
+#ifdef HAVE_RSVG
+gboolean
+eog_image_is_svg (EogImage *img)
+{
+	g_return_val_if_fail (EOG_IS_IMAGE (img), FALSE);
+
+	return (img->priv->svg != NULL);
+}
+
+RsvgHandle *
+eog_image_get_svg (EogImage *img)
+{
+	g_return_val_if_fail (EOG_IS_IMAGE (img), NULL);
+
+	return img->priv->svg;
+}
+
+EogTransform *
+eog_image_get_transform (EogImage *img)
+{
+	g_return_val_if_fail (EOG_IS_IMAGE (img), NULL);
+
+	return img->priv->trans;
+}
+
+#endif
+
+/**
+ * eog_image_file_changed:
+ * @img: a #EogImage
+ *
+ * Emits EogImage::file-changed signal
+ **/
+void
+eog_image_file_changed (EogImage *img)
+{
+	g_return_if_fail (EOG_IS_IMAGE (img));
+
+	g_signal_emit (img, signals[SIGNAL_FILE_CHANGED], 0);
+}
